"""The asyncio package, tracking PEP 3156."""

# This condition is so big, it's clearer to keep to platform condition in two blocks
# Can't NOQA on a specific line: https://github.com/plinss/flake8-noqa/issues/22
import sys
from collections.abc import Awaitable, Coroutine, Generator
from typing import Any, TypeVar
from typing_extensions import TypeAlias

# As at runtime, this depends on all submodules defining __all__ accurately.
from .base_events import *
from .coroutines import *
from .events import *
from .exceptions import *
from .futures import *
from .locks import *
from .protocols import *
from .queues import *
from .runners import *
from .streams import *
from .subprocess import *
from .tasks import *
from .threads import *
from .transports import *

if sys.version_info >= (3, 14):
    from .graph import *

if sys.version_info >= (3, 11):
    from .taskgroups import *
    from .timeouts import *

if sys.platform == "win32":
    from .windows_events import *
else:
    from .unix_events import *

if sys.platform == "win32":
    if sys.version_info >= (3, 14):

        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "future_discard_from_awaited_by",  # from futures
            "future_add_to_awaited_by",  # from futures
            "capture_call_graph",  # from graph
            "format_call_graph",  # from graph
            "print_call_graph",  # from graph
            "FrameCallGraphEntry",  # from graph
            "FutureCallGraph",  # from graph
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "QueueShutDown",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from windows_events
            "ProactorEventLoop",  # from windows_events
            "IocpProactor",  # from windows_events
            "_DefaultEventLoopPolicy",  # from windows_events
            "_WindowsSelectorEventLoopPolicy",  # from windows_events
            "_WindowsProactorEventLoopPolicy",  # from windows_events
            "EventLoop",  # from windows_events
        )
    elif sys.version_info >= (3, 13):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "QueueShutDown",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from windows_events
            "ProactorEventLoop",  # from windows_events
            "IocpProactor",  # from windows_events
            "DefaultEventLoopPolicy",  # from windows_events
            "WindowsSelectorEventLoopPolicy",  # from windows_events
            "WindowsProactorEventLoopPolicy",  # from windows_events
            "EventLoop",  # from windows_events
        )
    elif sys.version_info >= (3, 12):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from windows_events
            "ProactorEventLoop",  # from windows_events
            "IocpProactor",  # from windows_events
            "DefaultEventLoopPolicy",  # from windows_events
            "WindowsSelectorEventLoopPolicy",  # from windows_events
            "WindowsProactorEventLoopPolicy",  # from windows_events
        )
    elif sys.version_info >= (3, 11):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from windows_events
            "ProactorEventLoop",  # from windows_events
            "IocpProactor",  # from windows_events
            "DefaultEventLoopPolicy",  # from windows_events
            "WindowsSelectorEventLoopPolicy",  # from windows_events
            "WindowsProactorEventLoopPolicy",  # from windows_events
        )
    else:
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "coroutine",  # from coroutines
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "to_thread",  # from threads
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from windows_events
            "ProactorEventLoop",  # from windows_events
            "IocpProactor",  # from windows_events
            "DefaultEventLoopPolicy",  # from windows_events
            "WindowsSelectorEventLoopPolicy",  # from windows_events
            "WindowsProactorEventLoopPolicy",  # from windows_events
        )
else:
    if sys.version_info >= (3, 14):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "future_discard_from_awaited_by",  # from futures
            "future_add_to_awaited_by",  # from futures
            "capture_call_graph",  # from graph
            "format_call_graph",  # from graph
            "print_call_graph",  # from graph
            "FrameCallGraphEntry",  # from graph
            "FutureCallGraph",  # from graph
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "QueueShutDown",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "open_unix_connection",  # from streams
            "start_unix_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from unix_events
            "EventLoop",  # from unix_events
        )
    elif sys.version_info >= (3, 13):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "QueueShutDown",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "open_unix_connection",  # from streams
            "start_unix_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from unix_events
            "AbstractChildWatcher",  # from unix_events
            "SafeChildWatcher",  # from unix_events
            "FastChildWatcher",  # from unix_events
            "PidfdChildWatcher",  # from unix_events
            "MultiLoopChildWatcher",  # from unix_events
            "ThreadedChildWatcher",  # from unix_events
            "DefaultEventLoopPolicy",  # from unix_events
            "EventLoop",  # from unix_events
        )
    elif sys.version_info >= (3, 12):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "open_unix_connection",  # from streams
            "start_unix_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "create_eager_task_factory",  # from tasks
            "eager_task_factory",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "TaskGroup",  # from taskgroups
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from unix_events
            "AbstractChildWatcher",  # from unix_events
            "SafeChildWatcher",  # from unix_events
            "FastChildWatcher",  # from unix_events
            "PidfdChildWatcher",  # from unix_events
            "MultiLoopChildWatcher",  # from unix_events
            "ThreadedChildWatcher",  # from unix_events
            "DefaultEventLoopPolicy",  # from unix_events
        )
    elif sys.version_info >= (3, 11):
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "BrokenBarrierError",  # from exceptions
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "Barrier",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "Runner",  # from runners
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "open_unix_connection",  # from streams
            "start_unix_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "to_thread",  # from threads
            "Timeout",  # from timeouts
            "timeout",  # from timeouts
            "timeout_at",  # from timeouts
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from unix_events
            "AbstractChildWatcher",  # from unix_events
            "SafeChildWatcher",  # from unix_events
            "FastChildWatcher",  # from unix_events
            "PidfdChildWatcher",  # from unix_events
            "MultiLoopChildWatcher",  # from unix_events
            "ThreadedChildWatcher",  # from unix_events
            "DefaultEventLoopPolicy",  # from unix_events
        )
    else:
        __all__ = (
            "BaseEventLoop",  # from base_events
            "Server",  # from base_events
            "coroutine",  # from coroutines
            "iscoroutinefunction",  # from coroutines
            "iscoroutine",  # from coroutines
            "AbstractEventLoopPolicy",  # from events
            "AbstractEventLoop",  # from events
            "AbstractServer",  # from events
            "Handle",  # from events
            "TimerHandle",  # from events
            "get_event_loop_policy",  # from events
            "set_event_loop_policy",  # from events
            "get_event_loop",  # from events
            "set_event_loop",  # from events
            "new_event_loop",  # from events
            "get_child_watcher",  # from events
            "set_child_watcher",  # from events
            "_set_running_loop",  # from events
            "get_running_loop",  # from events
            "_get_running_loop",  # from events
            "CancelledError",  # from exceptions
            "InvalidStateError",  # from exceptions
            "TimeoutError",  # from exceptions
            "IncompleteReadError",  # from exceptions
            "LimitOverrunError",  # from exceptions
            "SendfileNotAvailableError",  # from exceptions
            "Future",  # from futures
            "wrap_future",  # from futures
            "isfuture",  # from futures
            "Lock",  # from locks
            "Event",  # from locks
            "Condition",  # from locks
            "Semaphore",  # from locks
            "BoundedSemaphore",  # from locks
            "BaseProtocol",  # from protocols
            "Protocol",  # from protocols
            "DatagramProtocol",  # from protocols
            "SubprocessProtocol",  # from protocols
            "BufferedProtocol",  # from protocols
            "run",  # from runners
            "Queue",  # from queues
            "PriorityQueue",  # from queues
            "LifoQueue",  # from queues
            "QueueFull",  # from queues
            "QueueEmpty",  # from queues
            "StreamReader",  # from streams
            "StreamWriter",  # from streams
            "StreamReaderProtocol",  # from streams
            "open_connection",  # from streams
            "start_server",  # from streams
            "open_unix_connection",  # from streams
            "start_unix_server",  # from streams
            "create_subprocess_exec",  # from subprocess
            "create_subprocess_shell",  # from subprocess
            "Task",  # from tasks
            "create_task",  # from tasks
            "FIRST_COMPLETED",  # from tasks
            "FIRST_EXCEPTION",  # from tasks
            "ALL_COMPLETED",  # from tasks
            "wait",  # from tasks
            "wait_for",  # from tasks
            "as_completed",  # from tasks
            "sleep",  # from tasks
            "gather",  # from tasks
            "shield",  # from tasks
            "ensure_future",  # from tasks
            "run_coroutine_threadsafe",  # from tasks
            "current_task",  # from tasks
            "all_tasks",  # from tasks
            "_register_task",  # from tasks
            "_unregister_task",  # from tasks
            "_enter_task",  # from tasks
            "_leave_task",  # from tasks
            "to_thread",  # from threads
            "BaseTransport",  # from transports
            "ReadTransport",  # from transports
            "WriteTransport",  # from transports
            "Transport",  # from transports
            "DatagramTransport",  # from transports
            "SubprocessTransport",  # from transports
            "SelectorEventLoop",  # from unix_events
            "AbstractChildWatcher",  # from unix_events
            "SafeChildWatcher",  # from unix_events
            "FastChildWatcher",  # from unix_events
            "PidfdChildWatcher",  # from unix_events
            "MultiLoopChildWatcher",  # from unix_events
            "ThreadedChildWatcher",  # from unix_events
            "DefaultEventLoopPolicy",  # from unix_events
        )

_T_co = TypeVar("_T_co", covariant=True)

# Aliases imported by multiple submodules in typeshed
if sys.version_info >= (3, 12):
    _AwaitableLike: TypeAlias = Awaitable[_T_co]  # noqa: Y047
    _CoroutineLike: TypeAlias = Coroutine[Any, Any, _T_co]  # noqa: Y047
else:
    _AwaitableLike: TypeAlias = Generator[Any, None, _T_co] | Awaitable[_T_co]
    _CoroutineLike: TypeAlias = Generator[Any, None, _T_co] | Coroutine[Any, Any, _T_co]



# ruff: noqa: PYI021
import sys
import types
from collections.abc import Iterable
from enum import Enum
from typing import Any, ClassVar, Protocol, _SpecialForm

from typing_extensions import LiteralString, Self  # noqa: UP035

# Special operations
def static_assert(condition: object, msg: LiteralString | None = None) -> None: ...

# Types
Unknown = object()
AlwaysTruthy = object()
AlwaysFalsy = object()

# Special forms
Not: _SpecialForm
Intersection: _SpecialForm
TypeOf: _SpecialForm
CallableTypeOf: _SpecialForm

Top: _SpecialForm
"""
`Top[T]` represents the "top materialization" of `T`.

For any type `T`, the top [materialization] of `T` is a type that is
a supertype of all materializations of `T`.

For a [fully static] type `T`, `Top[T]` is always exactly the same type
as `T` itself. For example, the top materialization of `Sequence[int]`
is simply `Sequence[int]`.

For a [gradual type] `T` that contains [`Any`][Any] or `Unknown` inside
it, however, `Top[T]` will not be equivalent to `T`. `Top[Sequence[Any]]`
evaluates to `Sequence[object]`: since `Sequence` is covariant, no
possible materialization of `Any` exists such that a fully static
materialization of `Sequence[Any]` would not be a subtype of
`Sequence[object]`.

`Top[T]` cannot be simplified further for invariant gradual types.
`Top[list[Any]]` cannot be simplified to any other type: because `list`
is invariant, `list[object]` is not a supertype of `list[int]`. The
top materialization of `list[Any]` is simply `Top[list[Any]]`: the
infinite union of `list[T]` for every possible fully static type `T`.

[materialization]: https://typing.python.org/en/latest/spec/concepts.html#materialization
[fully static]: https://typing.python.org/en/latest/spec/concepts.html#fully-static-types
[gradual type]: https://typing.python.org/en/latest/spec/concepts.html#gradual-types
[Any]: https://typing.python.org/en/latest/spec/special-types.html#any
"""

Bottom: _SpecialForm
"""
`Bottom[T]` represents the "bottom materialization" of `T`.

For any type `T`, the bottom [materialization] of `T` is a type that is
a subtype of all materializations of `T`.

For a [fully static] type `T`, `Bottom[T]` is always exactly the same type
as `T` itself. For example, the bottom materialization of `Sequence[int]`
is simply `Sequence[int]`.

For a [gradual type] `T` that contains [`Any`][Any] or `Unknown` inside it,
however, `Bottom[T]` will not be equivalent to `T`. `Bottom[Sequence[Any]]`
evaluates to `Sequence[Never]`: since `Sequence` is covariant, no
possible materialization of `Any` exists such that a fully static
materialization of `Sequence[Any]` would not be a supertype of
`Sequence[Never]`. (`Sequence[Never]` is not the same type as the
uninhabited type `Never`: for example, it is inhabited by the empty tuple,
`()`.)

For many invariant gradual types `T`, `Bottom[T]` is equivalent to
[`Never`][Never], although ty will not necessarily apply this simplification
eagerly.

[materialization]: https://typing.python.org/en/latest/spec/concepts.html#materialization
[fully static]: https://typing.python.org/en/latest/spec/concepts.html#fully-static-types
[gradual type]: https://typing.python.org/en/latest/spec/concepts.html#gradual-types
[Never]: https://typing.python.org/en/latest/spec/special-types.html#never
[Any]: https://typing.python.org/en/latest/spec/special-types.html#any
"""

# ty treats annotations of `float` to mean `float | int`, and annotations of `complex`
# to mean `complex | float | int`. This is to support a typing-system special case [1].
# We therefore provide `JustFloat` and `JustComplex` to represent the "bare" `float` and
# `complex` types, respectively.
#
# [1]: https://typing.readthedocs.io/en/latest/spec/special-types.html#special-cases-for-float-and-complex
type JustFloat = TypeOf[1.0]
type JustComplex = TypeOf[1.0j]

# Constraints
class ConstraintSet:
    @staticmethod
    def range(lower_bound: Any, typevar: Any, upper_bound: Any) -> Self:
        """
        Returns a constraint set that requires `typevar` to specialize to a type
        that is a supertype of `lower_bound` and a subtype of `upper_bound`.
        """

    @staticmethod
    def always() -> Self:
        """Returns a constraint set that is always satisfied"""

    @staticmethod
    def never() -> Self:
        """Returns a constraint set that is never satisfied"""

    def implies_subtype_of(self, ty: Any, of: Any) -> Self:
        """
        Returns a constraint set that is satisfied when `ty` is a `subtype`_ of
        `of`, assuming that all of the constraints in `self` hold.

        .. _subtype: https://typing.python.org/en/latest/spec/concepts.html#subtype-supertype-and-type-equivalence
        """

    def satisfies(self, other: Self) -> Self:
        """
        Returns whether this constraint set satisfies another — that is, whether
        every specialization that satisfies this constraint set also satisfies
        `other`.
        """

    def satisfied_by_all_typevars(
        self, *, inferable: tuple[Any, ...] | None = None
    ) -> bool:
        """
        Returns whether this constraint set is satisfied by all of the typevars
        that it mentions. You must provide a tuple of the typevars that should
        be considered `inferable`. All other typevars mentioned in the
        constraint set will be considered non-inferable.
        """

    def __bool__(self) -> bool: ...
    def __eq__(self, other: ConstraintSet) -> bool: ...
    def __ne__(self, other: ConstraintSet) -> bool: ...
    def __and__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __or__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __invert__(self) -> ConstraintSet: ...

class GenericContext:
    """
    The set of typevars that are bound by a generic class, function, or type
    alias.
    """

class Specialization:
    """A mapping of typevars to specific types"""

# Predicates on types
#
# Ideally, these would be annotated using `TypeForm`, but that has not been
# standardized yet (https://peps.python.org/pep-0747).
def is_equivalent_to(type_a: Any, type_b: Any) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `type_a` and `type_b` are
    `equivalent`_ types.

    .. _equivalent: https://typing.python.org/en/latest/spec/glossary.html#term-equivalent
    """

def is_subtype_of(ty: Any, of: Any) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `ty` is a `subtype`_ of `of`.

    .. _subtype: https://typing.python.org/en/latest/spec/concepts.html#subtype-supertype-and-type-equivalence
    """

def is_assignable_to(ty: Any, to: Any) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `ty` is `assignable`_ to `to`.

    .. _assignable: https://typing.python.org/en/latest/spec/concepts.html#the-assignable-to-or-consistent-subtyping-relation
    """

def is_disjoint_from(type_a: Any, type_b: Any) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `type_a` and `type_b` are disjoint types.

    Two types are disjoint if they have no inhabitants in common.
    """

def is_singleton(ty: Any) -> bool:
    """Returns `True` if `ty` is a singleton type with exactly one inhabitant."""

def is_single_valued(ty: Any) -> bool:
    """Returns `True` if `ty` is non-empty and all inhabitants compare equal to each other."""

def generic_context(ty: Any) -> GenericContext | None:
    """Returns the generic context of a type as a tuple of typevars.

    Returns `None` if the type is not generic.
    """

def into_callable(ty: Any) -> Any:
    """Converts a value into a `Callable`, if possible.

    This is the value equivalent of `CallableTypeOf`, which operates on types.
    """

def dunder_all_names(module: Any) -> tuple[LiteralString, ...] | None:
    """Returns the `__all__` names of a module as a tuple of sorted strings.

    Returns `None` if either the module does not have `__all__` or it has invalid elements.
    """

def enum_members[E: type[Enum]](enum: E) -> tuple[LiteralString, ...]:
    """List all members of an enum."""

def all_members(obj: Any) -> tuple[LiteralString, ...]:
    """Returns a tuple of all members of the given object.

    This nearly emulates `dir(obj)` and `inspect.getmembers(obj)` in Python's
    standard library, but has at least the following differences:

    * `dir` and `inspect.getmembers` may use runtime mutable state to construct
      the list of attributes returned. In contrast, this routine is limited to
      static information only.
    * `dir` will respect an object's `__dir__` implementation, if present, but
      this method (currently) does not.
    """

def has_member(obj: Any, name: LiteralString) -> bool:
    """Returns `True` if the given object has a member with the given name."""

def reveal_protocol_interface(protocol: type) -> None:
    """
    Passing a protocol type to this function will cause ty to emit an info-level
    diagnostic describing the protocol's interface.

    Passing a non-protocol type will cause ty to emit an error diagnostic.
    """

def reveal_mro(cls: type | types.GenericAlias) -> None:
    """Reveal the MRO that ty infers for the given class or generic alias."""

class NamedTupleLike(Protocol):
    """
    A protocol describing an interface that should be satisfied by all named tuples
    created using `typing.NamedTuple` or `collections.namedtuple`.
    """

    # _fields is defined as `tuple[Any, ...]` rather than `tuple[str, ...]` so
    # that instances of actual `NamedTuple` classes with more precise `_fields`
    # types are considered assignable to this protocol (protocol attribute members
    # are invariant, and `tuple[str, str]` is not invariantly assignable to
    # `tuple[str, ...]`).
    _fields: ClassVar[tuple[Any, ...]]
    _field_defaults: ClassVar[dict[str, Any]]
    @classmethod
    def _make(cls: type[Self], iterable: Iterable[Any]) -> Self: ...
    def _asdict(self, /) -> dict[str, Any]: ...

    # Positional arguments aren't actually accepted by these methods at runtime,
    # but adding the `*args` parameters means that all `NamedTuple` classes
    # are understood as assignable to this protocol due to the special case
    # outlined in https://typing.python.org/en/latest/spec/callables.html#meaning-of-in-callable:
    #
    # > If the input signature in a function definition includes both a
    # > `*args` and `**kwargs` parameter and both are typed as `Any`
    # > (explicitly or implicitly because it has no annotation), a type
    # > checker should treat this as the equivalent of `...`.
    def _replace(self, *args, **kwargs) -> Self: ...
    if sys.version_info >= (3, 13):
        def __replace__(self, *args, **kwargs) -> Self: ...



if True:
    if True:
        if True:
            return _(
                "qweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweas "
                + "qweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqwegqweasdzxcqweasdzxc.",
                "qweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqweasdzxcqwe",
            ) % {"reported_username": reported_username, "report_reason": report_reason}



"""distutils.archive_util

Utility functions for creating archive files (tarballs, zip files,
that sort of thing).
"""

from _typeshed import StrOrBytesPath, StrPath
from typing import Literal, overload

@overload
def make_archive(
    base_name: str,
    format: str,
    root_dir: StrOrBytesPath | None = None,
    base_dir: str | None = None,
    verbose: bool | Literal[0, 1] = 0,
    dry_run: bool | Literal[0, 1] = 0,
    owner: str | None = None,
    group: str | None = None,
) -> str:
    """Create an archive file (eg. zip or tar).

    'base_name' is the name of the file to create, minus any format-specific
    extension; 'format' is the archive format: one of "zip", "tar", "gztar",
    "bztar", "xztar", or "ztar".

    'root_dir' is a directory that will be the root directory of the
    archive; ie. we typically chdir into 'root_dir' before creating the
    archive.  'base_dir' is the directory where we start archiving from;
    ie. 'base_dir' will be the common prefix of all files and
    directories in the archive.  'root_dir' and 'base_dir' both default
    to the current directory.  Returns the name of the archive file.

    'owner' and 'group' are used when creating a tar archive. By default,
    uses the current owner and group.
    """

@overload
def make_archive(
    base_name: StrPath,
    format: str,
    root_dir: StrOrBytesPath,
    base_dir: str | None = None,
    verbose: bool | Literal[0, 1] = 0,
    dry_run: bool | Literal[0, 1] = 0,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
def make_tarball(
    base_name: str,
    base_dir: StrPath,
    compress: str | None = "gzip",
    verbose: bool | Literal[0, 1] = 0,
    dry_run: bool | Literal[0, 1] = 0,
    owner: str | None = None,
    group: str | None = None,
) -> str:
    """Create a (possibly compressed) tar file from all the files under
    'base_dir'.

    'compress' must be "gzip" (the default), "bzip2", "xz", "compress", or
    None.  ("compress" will be deprecated in Python 3.2)

    'owner' and 'group' can be used to define an owner and a group for the
    archive that is being built. If not provided, the current owner and group
    will be used.

    The output tar file will be named 'base_dir' +  ".tar", possibly plus
    the appropriate compression extension (".gz", ".bz2", ".xz" or ".Z").

    Returns the output filename.
    """

def make_zipfile(base_name: str, base_dir: str, verbose: bool | Literal[0, 1] = 0, dry_run: bool | Literal[0, 1] = 0) -> str:
    """Create a zip file from all the files under 'base_dir'.

    The output zip file will be named 'base_name' + ".zip".  Uses either the
    "zipfile" Python module (if available) or the InfoZIP "zip" utility
    (if installed and found on the default search path).  If neither tool is
    available, raises DistutilsExecError.  Returns the name of the output zip
    file.
    """



class MyClass:

    # Range that falls entirely in a suppressed range
    # fmt: off<RANGE_START>
    def method(  self  ):
        print   ( "str" )
    <RANGE_END># fmt: on

    # This should net get formatted because it isn't in a formatting range.
    def not_in_formatting_range ( self): ...


    # Range that starts in a suppressed range and ends in a formatting range
    # fmt: off<RANGE_START>
    def other(  self):
        print   ( "str" )

    # fmt: on

    def formatted  ( self):
        pass
    <RANGE_END>
    def outside_formatting_range (self): pass




def test ():
    print( "hello" )
    <RANGE_START># leading comment
    1  + 2

    print( "world" )<RANGE_END>

    print( "unformatted" )


print( "Hy" )


def test2 ():
    print( "hello" )
    # Leading comments don't get formatted. That's why Ruff won't fixup
    # the indentation here. That's something we might want to explore in the future
    # leading comment 1<RANGE_START>
        # leading comment 2
    1  + 2

    print( "world" )<RANGE_END>

    print( "unformatted" )

def test3 ():
    <RANGE_START>print( "hello" )
    # leading comment 1
        # leading comment 2
    1  + 2

    print( "world" )<RANGE_END>

    print( "unformatted" )



"""Test result object"""

import sys
import unittest.case
from _typeshed import OptExcInfo
from collections.abc import Callable
from typing import Any, Final, TextIO, TypeVar
from typing_extensions import TypeAlias

_F = TypeVar("_F", bound=Callable[..., Any])
_DurationsType: TypeAlias = list[tuple[str, float]]

STDOUT_LINE: Final[str]
STDERR_LINE: Final[str]

# undocumented
def failfast(method: _F) -> _F: ...

class TestResult:
    """Holder for test result information.

    Test results are automatically managed by the TestCase and TestSuite
    classes, and do not need to be explicitly manipulated by writers of tests.

    Each instance holds the total number of tests run, and collections of
    failures and errors that occurred among those test runs. The collections
    contain tuples of (testcase, exceptioninfo), where exceptioninfo is the
    formatted traceback of the error that occurred.
    """

    errors: list[tuple[unittest.case.TestCase, str]]
    failures: list[tuple[unittest.case.TestCase, str]]
    skipped: list[tuple[unittest.case.TestCase, str]]
    expectedFailures: list[tuple[unittest.case.TestCase, str]]
    unexpectedSuccesses: list[unittest.case.TestCase]
    shouldStop: bool
    testsRun: int
    buffer: bool
    failfast: bool
    tb_locals: bool
    if sys.version_info >= (3, 12):
        collectedDurations: _DurationsType

    def __init__(self, stream: TextIO | None = None, descriptions: bool | None = None, verbosity: int | None = None) -> None: ...
    def printErrors(self) -> None:
        """Called by TestRunner after test run"""

    def wasSuccessful(self) -> bool:
        """Tells whether or not this result was a success."""

    def stop(self) -> None:
        """Indicates that the tests should be aborted."""

    def startTest(self, test: unittest.case.TestCase) -> None:
        """Called when the given test is about to be run"""

    def stopTest(self, test: unittest.case.TestCase) -> None:
        """Called when the given test has been run"""

    def startTestRun(self) -> None:
        """Called once before any tests are executed.

        See startTest for a method called before each test.
        """

    def stopTestRun(self) -> None:
        """Called once after all tests are executed.

        See stopTest for a method called after each test.
        """

    def addError(self, test: unittest.case.TestCase, err: OptExcInfo) -> None:
        """Called when an error has occurred. 'err' is a tuple of values as
        returned by sys.exc_info().
        """

    def addFailure(self, test: unittest.case.TestCase, err: OptExcInfo) -> None:
        """Called when an error has occurred. 'err' is a tuple of values as
        returned by sys.exc_info().
        """

    def addSuccess(self, test: unittest.case.TestCase) -> None:
        """Called when a test has completed successfully"""

    def addSkip(self, test: unittest.case.TestCase, reason: str) -> None:
        """Called when a test is skipped."""

    def addExpectedFailure(self, test: unittest.case.TestCase, err: OptExcInfo) -> None:
        """Called when an expected failure/error occurred."""

    def addUnexpectedSuccess(self, test: unittest.case.TestCase) -> None:
        """Called when a test was expected to fail, but succeed."""

    def addSubTest(self, test: unittest.case.TestCase, subtest: unittest.case.TestCase, err: OptExcInfo | None) -> None:
        """Called at the end of a subtest.
        'err' is None if the subtest ended successfully, otherwise it's a
        tuple of values as returned by sys.exc_info().
        """
    if sys.version_info >= (3, 12):
        def addDuration(self, test: unittest.case.TestCase, elapsed: float) -> None:
            """Called when a test finished to run, regardless of its outcome.
            *test* is the test case corresponding to the test method.
            *elapsed* is the time represented in seconds, and it includes the
            execution of cleanup functions.
            """



x = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa

x_aa = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
xxxxx = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa

while (
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
):
    pass

while aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa:
    pass

# Only applies in `Parenthesize::IfBreaks` positions
raise aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa

raise (
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
)

raise a from aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
raise a from aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa

# Can never apply on expression statement level
aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa

# Is it only relevant for items that can't break

aaaaaaa = 111111111111111111111111111111111111111111111111111111111111111111111111111111
aaaaaaa = (
    1111111111111111111111111111111111111111111111111111111111111111111111111111111
)

aaaaaaa = """111111111111111111111111111111111111111111111111111111111111111111111111111
1111111111111111111111111111111111111111111111111111111111111111111111111111111111111"""

# Never parenthesize multiline strings
aaaaaaa = (
    """1111111111111111111111111111111111111111111111111111111111111111111111111111
1111111111111111111111111111111111111111111111111111111111111111111111111111111111111"""
)



aaaaaaaa = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbb
aaaaaaaa = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb

aaaaaaaa = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb


for converter in connection.ops.get_db_converters(
    expression
) + expression.get_db_converters(connection):
    pass


aaa = (
    bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb # awkward comment
)

def test():
    m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz = models.ManyToManyField(Person, blank=True)

    m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz = models.ManyToManyFieldAttributeChainField

m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz = models.ManyToManyField(Person, blank=True)
m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz = models.ManyToManyFieldAttributeChainFieeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeld

def test():
    if True:
        VLM_m2m = VLM.m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz.through
        allows_group_by_select_index = self.connection.features.allows_group_by_select_index


# This is a deviation from Black:
# Black keeps the comment inside of the parentheses, making it more likely to exceed the line width.
# Ruff renders the comment after the parentheses, giving it more space to fit.
if True:
    if True:
        if True:
            # Black layout
            model.config.use_cache = (
                False  # FSTM still requires this hack -> FSTM should probably be refactored s
            )
            # Ruff layout
            model.config.use_cache = False  # FSTM still requires this hack -> FSTM should probably be refactored s


# Regression test for https://github.com/astral-sh/ruff/issues/7463
mp3fmt="<span style=\"color: grey\"><a href=\"{}\" id=\"audiolink\">listen</a></span></br>\n"

# Regression test for https://github.com/astral-sh/ruff/issues/7067
def f():
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa = (
        True
    )



"""Class representing image/* type MIME documents."""

from collections.abc import Callable
from email import _ParamsType
from email.mime.nonmultipart import MIMENonMultipart
from email.policy import Policy

__all__ = ["MIMEImage"]

class MIMEImage(MIMENonMultipart):
    """Class for generating image/* type MIME documents."""

    def __init__(
        self,
        _imagedata: str | bytes | bytearray,
        _subtype: str | None = None,
        _encoder: Callable[[MIMEImage], object] = ...,
        *,
        policy: Policy | None = None,
        **_params: _ParamsType,
    ) -> None:
        """Create an image/* type MIME document.

        _imagedata contains the bytes for the raw image data.  If the data
        type can be detected (jpeg, png, gif, tiff, rgb, pbm, pgm, ppm,
        rast, xbm, bmp, webp, and exr attempted), then the subtype will be
        automatically included in the Content-Type header. Otherwise, you can
        specify the specific image subtype via the _subtype parameter.

        _encoder is a function which will perform the actual encoding for
        transport of the image data.  It takes one argument, which is this
        Image instance.  It should use get_payload() and set_payload() to
        change the payload to the encoded form.  It should also add any
        Content-Transfer-Encoding or other headers to the message as
        necessary.  The default encoding is Base64.

        Any additional keyword arguments are passed to the base class
        constructor, which turns them into parameters on the Content-Type
        header.
        """



# parse_options: {"target-version": "3.14"}
def f() -> (y := 3): ...
def g(arg: (x := 1)): ...
def outer():
    def i(x: (yield 1)): ...
    def k() -> (yield 1): ...
    def m(x: (yield from 1)): ...
    def o() -> (yield from 1): ...
async def outer():
    def f() -> (await 1): ...
    def g(arg: (await 1)): ...



import _tkinter
import itertools
import sys
import tkinter
from typing import Any, ClassVar, Final, Literal, TypedDict, overload, type_check_only
from typing_extensions import TypeAlias, Unpack

__all__ = ["NORMAL", "ROMAN", "BOLD", "ITALIC", "nametofont", "Font", "families", "names"]

NORMAL: Final = "normal"
ROMAN: Final = "roman"
BOLD: Final = "bold"
ITALIC: Final = "italic"

_FontDescription: TypeAlias = (
    str  # "Helvetica 12"
    | Font  # A font object constructed in Python
    | list[Any]  # ["Helvetica", 12, BOLD]
    | tuple[str]  # ("Liberation Sans",) needs wrapping in tuple/list to handle spaces
    # ("Liberation Sans", 12) or ("Liberation Sans", 12, "bold", "italic", "underline")
    | tuple[str, int, Unpack[tuple[str, ...]]]  # Any number of trailing options is permitted
    | tuple[str, int, list[str] | tuple[str, ...]]  # Options can also be passed as list/tuple
    | _tkinter.Tcl_Obj  # A font object constructed in Tcl
)

@type_check_only
class _FontDict(TypedDict):
    family: str
    size: int
    weight: Literal["normal", "bold"]
    slant: Literal["roman", "italic"]
    underline: bool
    overstrike: bool

@type_check_only
class _MetricsDict(TypedDict):
    ascent: int
    descent: int
    linespace: int
    fixed: bool

class Font:
    """Represents a named font.

    Constructor options are:

    font -- font specifier (name, system font, or (family, size, style)-tuple)
    name -- name to use for this font configuration (defaults to a unique name)
    exists -- does a named font by this name already exist?
       Creates a new named font if False, points to the existing font if True.
       Raises _tkinter.TclError if the assertion is false.

       the following are ignored if font is specified:

    family -- font 'family', e.g. Courier, Times, Helvetica
    size -- font size in points
    weight -- font thickness: NORMAL, BOLD
    slant -- font slant: ROMAN, ITALIC
    underline -- font underlining: false (0), true (1)
    overstrike -- font strikeout: false (0), true (1)

    """

    name: str
    delete_font: bool
    counter: ClassVar[itertools.count[int]]  # undocumented
    def __init__(
        self,
        # In tkinter, 'root' refers to tkinter.Tk by convention, but the code
        # actually works with any tkinter widget so we use tkinter.Misc.
        root: tkinter.Misc | None = None,
        font: _FontDescription | None = None,
        name: str | None = None,
        exists: bool = False,
        *,
        family: str = ...,
        size: int = ...,
        weight: Literal["normal", "bold"] = ...,
        slant: Literal["roman", "italic"] = ...,
        underline: bool = ...,
        overstrike: bool = ...,
    ) -> None: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __setitem__(self, key: str, value: Any) -> None: ...
    @overload
    def cget(self, option: Literal["family"]) -> str:
        """Get font attribute"""

    @overload
    def cget(self, option: Literal["size"]) -> int: ...
    @overload
    def cget(self, option: Literal["weight"]) -> Literal["normal", "bold"]: ...
    @overload
    def cget(self, option: Literal["slant"]) -> Literal["roman", "italic"]: ...
    @overload
    def cget(self, option: Literal["underline", "overstrike"]) -> bool: ...
    @overload
    def cget(self, option: str) -> Any: ...
    __getitem__ = cget
    @overload
    def actual(self, option: Literal["family"], displayof: tkinter.Misc | None = None) -> str:
        """Return actual font attributes"""

    @overload
    def actual(self, option: Literal["size"], displayof: tkinter.Misc | None = None) -> int: ...
    @overload
    def actual(self, option: Literal["weight"], displayof: tkinter.Misc | None = None) -> Literal["normal", "bold"]: ...
    @overload
    def actual(self, option: Literal["slant"], displayof: tkinter.Misc | None = None) -> Literal["roman", "italic"]: ...
    @overload
    def actual(self, option: Literal["underline", "overstrike"], displayof: tkinter.Misc | None = None) -> bool: ...
    @overload
    def actual(self, option: None, displayof: tkinter.Misc | None = None) -> _FontDict: ...
    @overload
    def actual(self, *, displayof: tkinter.Misc | None = None) -> _FontDict: ...
    def config(
        self,
        *,
        family: str = ...,
        size: int = ...,
        weight: Literal["normal", "bold"] = ...,
        slant: Literal["roman", "italic"] = ...,
        underline: bool = ...,
        overstrike: bool = ...,
    ) -> _FontDict | None:
        """Modify font attributes"""
    configure = config
    def copy(self) -> Font:
        """Return a distinct copy of the current font"""

    @overload
    def metrics(self, option: Literal["ascent", "descent", "linespace"], /, *, displayof: tkinter.Misc | None = ...) -> int:
        """Return font metrics.

        For best performance, create a dummy widget
        using this font before calling this method.
        """

    @overload
    def metrics(self, option: Literal["fixed"], /, *, displayof: tkinter.Misc | None = ...) -> bool: ...
    @overload
    def metrics(self, *, displayof: tkinter.Misc | None = ...) -> _MetricsDict: ...
    def measure(self, text: str, displayof: tkinter.Misc | None = None) -> int:
        """Return text width"""

    def __eq__(self, other: object) -> bool: ...
    def __del__(self) -> None: ...

def families(root: tkinter.Misc | None = None, displayof: tkinter.Misc | None = None) -> tuple[str, ...]:
    """Get font families (as a tuple)"""

def names(root: tkinter.Misc | None = None) -> tuple[str, ...]:
    """Get names of defined fonts (as a tuple)"""

if sys.version_info >= (3, 10):
    def nametofont(name: str, root: tkinter.Misc | None = None) -> Font:
        """Given the name of a tk named font, returns a Font representation."""

else:
    def nametofont(name: str) -> Font:
        """Given the name of a tk named font, returns a Font representation."""



"""Simple textbox editing widget with Emacs-like keybindings."""

from _curses import window
from collections.abc import Callable

def rectangle(win: window, uly: int, ulx: int, lry: int, lrx: int) -> None:
    """Draw a rectangle with corners at the provided upper-left
    and lower-right coordinates.
    """

class Textbox:
    """Editing widget using the interior of a window object.
     Supports the following Emacs-like key bindings:

    Ctrl-A      Go to left edge of window.
    Ctrl-B      Cursor left, wrapping to previous line if appropriate.
    Ctrl-D      Delete character under cursor.
    Ctrl-E      Go to right edge (stripspaces off) or end of line (stripspaces on).
    Ctrl-F      Cursor right, wrapping to next line when appropriate.
    Ctrl-G      Terminate, returning the window contents.
    Ctrl-H      Delete character backward.
    Ctrl-J      Terminate if the window is 1 line, otherwise insert newline.
    Ctrl-K      If line is blank, delete it, otherwise clear to end of line.
    Ctrl-L      Refresh screen.
    Ctrl-N      Cursor down; move down one line.
    Ctrl-O      Insert a blank line at cursor location.
    Ctrl-P      Cursor up; move up one line.

    Move operations do nothing if the cursor is at an edge where the movement
    is not possible.  The following synonyms are supported where possible:

    KEY_LEFT = Ctrl-B, KEY_RIGHT = Ctrl-F, KEY_UP = Ctrl-P, KEY_DOWN = Ctrl-N
    KEY_BACKSPACE = Ctrl-h
    """

    stripspaces: bool
    def __init__(self, win: window, insert_mode: bool = False) -> None: ...
    def edit(self, validate: Callable[[int], int] | None = None) -> str:
        """Edit in the widget window and collect the results."""

    def do_command(self, ch: str | int) -> None:
        """Process a single editing command."""

    def gather(self) -> str:
        """Collect and return the contents of the window."""



# Test cases for set expressions where the parser recovers from a syntax error.
# There are valid expressions in between invalid ones to verify that.
# These are same as for the list expressions.

{,}

{1,,2}

{1,,}

# Missing comma
{1 2}

# Dictionary element in a list
{1: 2}

# Missing expression
{1, x + }

{1; 2}

[*]



"""Read/write support for Maildir, mbox, MH, Babyl, and MMDF mailboxes."""

import email.message
import io
import sys
from _typeshed import StrPath, SupportsNoArgReadline, SupportsRead
from abc import ABCMeta, abstractmethod
from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
from email._policybase import _MessageT
from types import GenericAlias, TracebackType
from typing import IO, Any, AnyStr, Generic, Literal, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias

__all__ = [
    "Mailbox",
    "Maildir",
    "mbox",
    "MH",
    "Babyl",
    "MMDF",
    "Message",
    "MaildirMessage",
    "mboxMessage",
    "MHMessage",
    "BabylMessage",
    "MMDFMessage",
    "Error",
    "NoSuchMailboxError",
    "NotEmptyError",
    "ExternalClashError",
    "FormatError",
]

_T = TypeVar("_T")

@type_check_only
class _SupportsReadAndReadline(SupportsRead[bytes], SupportsNoArgReadline[bytes], Protocol): ...

_MessageData: TypeAlias = email.message.Message | bytes | str | io.StringIO | _SupportsReadAndReadline

@type_check_only
class _HasIteritems(Protocol):
    def iteritems(self) -> Iterator[tuple[str, _MessageData]]: ...

@type_check_only
class _HasItems(Protocol):
    def items(self) -> Iterator[tuple[str, _MessageData]]: ...

linesep: bytes

class Mailbox(Generic[_MessageT]):
    """A group of messages in a particular place."""

    _path: str  # undocumented
    _factory: Callable[[IO[Any]], _MessageT] | None  # undocumented
    @overload
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], _MessageT], create: bool = True) -> None:
        """Initialize a Mailbox instance."""

    @overload
    def __init__(self, path: StrPath, factory: None = None, create: bool = True) -> None: ...
    @abstractmethod
    def add(self, message: _MessageData) -> str:
        """Add message and return assigned key."""

    @abstractmethod
    def remove(self, key: str) -> None:
        """Remove the keyed message; raise KeyError if it doesn't exist."""

    def __delitem__(self, key: str) -> None: ...
    def discard(self, key: str) -> None:
        """If the keyed message exists, remove it."""

    @abstractmethod
    def __setitem__(self, key: str, message: _MessageData) -> None:
        """Replace the keyed message; raise KeyError if it doesn't exist."""

    @overload
    def get(self, key: str, default: None = None) -> _MessageT | None:
        """Return the keyed message, or default if it doesn't exist."""

    @overload
    def get(self, key: str, default: _T) -> _MessageT | _T: ...
    def __getitem__(self, key: str) -> _MessageT:
        """Return the keyed message; raise KeyError if it doesn't exist."""

    @abstractmethod
    def get_message(self, key: str) -> _MessageT:
        """Return a Message representation or raise a KeyError."""

    def get_string(self, key: str) -> str:
        """Return a string representation or raise a KeyError.

        Uses email.message.Message to create a 7bit clean string
        representation of the message.
        """

    @abstractmethod
    def get_bytes(self, key: str) -> bytes:
        """Return a byte string representation or raise a KeyError."""
    # As '_ProxyFile' doesn't implement the full IO spec, and BytesIO is incompatible with it, get_file return is Any here
    @abstractmethod
    def get_file(self, key: str) -> Any:
        """Return a file-like representation or raise a KeyError."""

    @abstractmethod
    def iterkeys(self) -> Iterator[str]:
        """Return an iterator over keys."""

    def keys(self) -> list[str]:
        """Return a list of keys."""

    def itervalues(self) -> Iterator[_MessageT]:
        """Return an iterator over all messages."""

    def __iter__(self) -> Iterator[_MessageT]: ...
    def values(self) -> list[_MessageT]:
        """Return a list of messages. Memory intensive."""

    def iteritems(self) -> Iterator[tuple[str, _MessageT]]:
        """Return an iterator over (key, message) tuples."""

    def items(self) -> list[tuple[str, _MessageT]]:
        """Return a list of (key, message) tuples. Memory intensive."""

    @abstractmethod
    def __contains__(self, key: str) -> bool:
        """Return True if the keyed message exists, False otherwise."""

    @abstractmethod
    def __len__(self) -> int:
        """Return a count of messages in the mailbox."""

    def clear(self) -> None:
        """Delete all messages."""

    @overload
    def pop(self, key: str, default: None = None) -> _MessageT | None:
        """Delete the keyed message and return it, or default."""

    @overload
    def pop(self, key: str, default: _T) -> _MessageT | _T: ...
    def popitem(self) -> tuple[str, _MessageT]:
        """Delete an arbitrary (key, message) pair and return it."""

    def update(self, arg: _HasIteritems | _HasItems | Iterable[tuple[str, _MessageData]] | None = None) -> None:
        """Change the messages that correspond to certain keys."""

    @abstractmethod
    def flush(self) -> None:
        """Write any pending changes to the disk."""

    @abstractmethod
    def lock(self) -> None:
        """Lock the mailbox."""

    @abstractmethod
    def unlock(self) -> None:
        """Unlock the mailbox if it is locked."""

    @abstractmethod
    def close(self) -> None:
        """Flush and close the mailbox."""

    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """Represent a PEP 585 generic type

        E.g. for t = list[int], t.__origin__ is list and t.__args__ is (int,).
        """

class Maildir(Mailbox[MaildirMessage]):
    """A qmail-style Maildir mailbox."""

    colon: str
    def __init__(self, dirname: StrPath, factory: Callable[[IO[Any]], MaildirMessage] | None = None, create: bool = True) -> None:
        """Initialize a Maildir instance."""

    def add(self, message: _MessageData) -> str:
        """Add message and return assigned key."""

    def remove(self, key: str) -> None:
        """Remove the keyed message; raise KeyError if it doesn't exist."""

    def __setitem__(self, key: str, message: _MessageData) -> None:
        """Replace the keyed message; raise KeyError if it doesn't exist."""

    def get_message(self, key: str) -> MaildirMessage:
        """Return a Message representation or raise a KeyError."""

    def get_bytes(self, key: str) -> bytes:
        """Return a bytes representation or raise a KeyError."""

    def get_file(self, key: str) -> _ProxyFile[bytes]:
        """Return a file-like representation or raise a KeyError."""
    if sys.version_info >= (3, 13):
        def get_info(self, key: str) -> str:
            """Get the keyed message's "info" as a string."""

        def set_info(self, key: str, info: str) -> None:
            """Set the keyed message's "info" string."""

        def get_flags(self, key: str) -> str:
            """Return as a string the standard flags that are set on the keyed message."""

        def set_flags(self, key: str, flags: str) -> None:
            """Set the given flags and unset all others on the keyed message."""

        def add_flag(self, key: str, flag: str) -> None:
            """Set the given flag(s) without changing others on the keyed message."""

        def remove_flag(self, key: str, flag: str) -> None:
            """Unset the given string flag(s) without changing others on the keyed message."""

    def iterkeys(self) -> Iterator[str]:
        """Return an iterator over keys."""

    def __contains__(self, key: str) -> bool:
        """Return True if the keyed message exists, False otherwise."""

    def __len__(self) -> int:
        """Return a count of messages in the mailbox."""

    def flush(self) -> None:
        """Write any pending changes to disk."""

    def lock(self) -> None:
        """Lock the mailbox."""

    def unlock(self) -> None:
        """Unlock the mailbox if it is locked."""

    def close(self) -> None:
        """Flush and close the mailbox."""

    def list_folders(self) -> list[str]:
        """Return a list of folder names."""

    def get_folder(self, folder: str) -> Maildir:
        """Return a Maildir instance for the named folder."""

    def add_folder(self, folder: str) -> Maildir:
        """Create a folder and return a Maildir instance representing it."""

    def remove_folder(self, folder: str) -> None:
        """Delete the named folder, which must be empty."""

    def clean(self) -> None:
        """Delete old files in "tmp"."""

    def next(self) -> str | None:
        """Return the next message in a one-time iteration."""

class _singlefileMailbox(Mailbox[_MessageT], metaclass=ABCMeta):
    """A single-file mailbox."""

    def add(self, message: _MessageData) -> str:
        """Add message and return assigned key."""

    def remove(self, key: str) -> None:
        """Remove the keyed message; raise KeyError if it doesn't exist."""

    def __setitem__(self, key: str, message: _MessageData) -> None:
        """Replace the keyed message; raise KeyError if it doesn't exist."""

    def iterkeys(self) -> Iterator[str]:
        """Return an iterator over keys."""

    def __contains__(self, key: str) -> bool:
        """Return True if the keyed message exists, False otherwise."""

    def __len__(self) -> int:
        """Return a count of messages in the mailbox."""

    def lock(self) -> None:
        """Lock the mailbox."""

    def unlock(self) -> None:
        """Unlock the mailbox if it is locked."""

    def flush(self) -> None:
        """Write any pending changes to disk."""

    def close(self) -> None:
        """Flush and close the mailbox."""

class _mboxMMDF(_singlefileMailbox[_MessageT]):
    """An mbox or MMDF mailbox."""

    def get_message(self, key: str) -> _MessageT:
        """Return a Message representation or raise a KeyError."""

    def get_file(self, key: str, from_: bool = False) -> _PartialFile[bytes]:
        """Return a file-like representation or raise a KeyError."""

    def get_bytes(self, key: str, from_: bool = False) -> bytes:
        """Return a string representation or raise a KeyError."""

    def get_string(self, key: str, from_: bool = False) -> str:
        """Return a string representation or raise a KeyError."""

class mbox(_mboxMMDF[mboxMessage]):
    """A classic mbox mailbox."""

    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], mboxMessage] | None = None, create: bool = True) -> None:
        """Initialize an mbox mailbox."""

class MMDF(_mboxMMDF[MMDFMessage]):
    """An MMDF mailbox."""

    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], MMDFMessage] | None = None, create: bool = True) -> None:
        """Initialize an MMDF mailbox."""

class MH(Mailbox[MHMessage]):
    """An MH mailbox."""

    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], MHMessage] | None = None, create: bool = True) -> None:
        """Initialize an MH instance."""

    def add(self, message: _MessageData) -> str:
        """Add message and return assigned key."""

    def remove(self, key: str) -> None:
        """Remove the keyed message; raise KeyError if it doesn't exist."""

    def __setitem__(self, key: str, message: _MessageData) -> None:
        """Replace the keyed message; raise KeyError if it doesn't exist."""

    def get_message(self, key: str) -> MHMessage:
        """Return a Message representation or raise a KeyError."""

    def get_bytes(self, key: str) -> bytes:
        """Return a bytes representation or raise a KeyError."""

    def get_file(self, key: str) -> _ProxyFile[bytes]:
        """Return a file-like representation or raise a KeyError."""

    def iterkeys(self) -> Iterator[str]:
        """Return an iterator over keys."""

    def __contains__(self, key: str) -> bool:
        """Return True if the keyed message exists, False otherwise."""

    def __len__(self) -> int:
        """Return a count of messages in the mailbox."""

    def flush(self) -> None:
        """Write any pending changes to the disk."""

    def lock(self) -> None:
        """Lock the mailbox."""

    def unlock(self) -> None:
        """Unlock the mailbox if it is locked."""

    def close(self) -> None:
        """Flush and close the mailbox."""

    def list_folders(self) -> list[str]:
        """Return a list of folder names."""

    def get_folder(self, folder: StrPath) -> MH:
        """Return an MH instance for the named folder."""

    def add_folder(self, folder: StrPath) -> MH:
        """Create a folder and return an MH instance representing it."""

    def remove_folder(self, folder: StrPath) -> None:
        """Delete the named folder, which must be empty."""

    def get_sequences(self) -> dict[str, list[int]]:
        """Return a name-to-key-list dictionary to define each sequence."""

    def set_sequences(self, sequences: Mapping[str, Sequence[int]]) -> None:
        """Set sequences using the given name-to-key-list dictionary."""

    def pack(self) -> None:
        """Re-name messages to eliminate numbering gaps. Invalidates keys."""

class Babyl(_singlefileMailbox[BabylMessage]):
    """An Rmail-style Babyl mailbox."""

    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], BabylMessage] | None = None, create: bool = True) -> None:
        """Initialize a Babyl mailbox."""

    def get_message(self, key: str) -> BabylMessage:
        """Return a Message representation or raise a KeyError."""

    def get_bytes(self, key: str) -> bytes:
        """Return a string representation or raise a KeyError."""

    def get_file(self, key: str) -> IO[bytes]:
        """Return a file-like representation or raise a KeyError."""

    def get_labels(self) -> list[str]:
        """Return a list of user-defined labels in the mailbox."""

class Message(email.message.Message):
    """Message with mailbox-format-specific properties."""

    def __init__(self, message: _MessageData | None = None) -> None:
        """Initialize a Message instance."""

class MaildirMessage(Message):
    """Message with Maildir-specific properties."""

    def get_subdir(self) -> str:
        """Return 'new' or 'cur'."""

    def set_subdir(self, subdir: Literal["new", "cur"]) -> None:
        """Set subdir to 'new' or 'cur'."""

    def get_flags(self) -> str:
        """Return as a string the flags that are set."""

    def set_flags(self, flags: Iterable[str]) -> None:
        """Set the given flags and unset all others."""

    def add_flag(self, flag: str) -> None:
        """Set the given flag(s) without changing others."""

    def remove_flag(self, flag: str) -> None:
        """Unset the given string flag(s) without changing others."""

    def get_date(self) -> int:
        """Return delivery date of message, in seconds since the epoch."""

    def set_date(self, date: float) -> None:
        """Set delivery date of message, in seconds since the epoch."""

    def get_info(self) -> str:
        """Get the message's "info" as a string."""

    def set_info(self, info: str) -> None:
        """Set the message's "info" string."""

class _mboxMMDFMessage(Message):
    """Message with mbox- or MMDF-specific properties."""

    def get_from(self) -> str:
        """Return contents of "From " line."""

    def set_from(self, from_: str, time_: bool | tuple[int, int, int, int, int, int, int, int, int] | None = None) -> None:
        """Set "From " line, formatting and appending time_ if specified."""

    def get_flags(self) -> str:
        """Return as a string the flags that are set."""

    def set_flags(self, flags: Iterable[str]) -> None:
        """Set the given flags and unset all others."""

    def add_flag(self, flag: str) -> None:
        """Set the given flag(s) without changing others."""

    def remove_flag(self, flag: str) -> None:
        """Unset the given string flag(s) without changing others."""

class mboxMessage(_mboxMMDFMessage):
    """Message with mbox-specific properties."""

class MHMessage(Message):
    """Message with MH-specific properties."""

    def get_sequences(self) -> list[str]:
        """Return a list of sequences that include the message."""

    def set_sequences(self, sequences: Iterable[str]) -> None:
        """Set the list of sequences that include the message."""

    def add_sequence(self, sequence: str) -> None:
        """Add sequence to list of sequences including the message."""

    def remove_sequence(self, sequence: str) -> None:
        """Remove sequence from the list of sequences including the message."""

class BabylMessage(Message):
    """Message with Babyl-specific properties."""

    def get_labels(self) -> list[str]:
        """Return a list of labels on the message."""

    def set_labels(self, labels: Iterable[str]) -> None:
        """Set the list of labels on the message."""

    def add_label(self, label: str) -> None:
        """Add label to list of labels on the message."""

    def remove_label(self, label: str) -> None:
        """Remove label from the list of labels on the message."""

    def get_visible(self) -> Message:
        """Return a Message representation of visible headers."""

    def set_visible(self, visible: _MessageData) -> None:
        """Set the Message representation of visible headers."""

    def update_visible(self) -> None:
        """Update and/or sensibly generate a set of visible headers."""

class MMDFMessage(_mboxMMDFMessage):
    """Message with MMDF-specific properties."""

class _ProxyFile(Generic[AnyStr]):
    """A read-only wrapper of a file."""

    def __init__(self, f: IO[AnyStr], pos: int | None = None) -> None:
        """Initialize a _ProxyFile."""

    def read(self, size: int | None = None) -> AnyStr:
        """Read bytes."""

    def read1(self, size: int | None = None) -> AnyStr:
        """Read bytes."""

    def readline(self, size: int | None = None) -> AnyStr:
        """Read a line."""

    def readlines(self, sizehint: int | None = None) -> list[AnyStr]:
        """Read multiple lines."""

    def __iter__(self) -> Iterator[AnyStr]:
        """Iterate over lines."""

    def tell(self) -> int:
        """Return the position."""

    def seek(self, offset: int, whence: int = 0) -> None:
        """Change position."""

    def close(self) -> None:
        """Close the file."""

    def __enter__(self) -> Self:
        """Context management protocol support."""

    def __exit__(self, exc_type: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def flush(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """Represent a PEP 585 generic type

        E.g. for t = list[int], t.__origin__ is list and t.__args__ is (int,).
        """

class _PartialFile(_ProxyFile[AnyStr]):
    """A read-only wrapper of part of a file."""

    def __init__(self, f: IO[AnyStr], start: int | None = None, stop: int | None = None) -> None:
        """Initialize a _PartialFile."""

class Error(Exception):
    """Raised for module-specific errors."""

class NoSuchMailboxError(Error):
    """The specified mailbox does not exist and won't be created."""

class NotEmptyError(Error):
    """The specified mailbox is not empty and deletion was requested."""

class ExternalClashError(Error):
    """Another process caused an action to fail."""

class FormatError(Error):
    """A file appears to have an invalid format."""



"""Optimized C implementation for the Python pickle module."""

from _typeshed import ReadableBuffer, SupportsWrite
from collections.abc import Callable, Iterable, Iterator, Mapping
from pickle import PickleBuffer as PickleBuffer
from typing import Any, Protocol, type_check_only
from typing_extensions import TypeAlias, disjoint_base

@type_check_only
class _ReadableFileobj(Protocol):
    def read(self, n: int, /) -> bytes: ...
    def readline(self) -> bytes: ...

_BufferCallback: TypeAlias = Callable[[PickleBuffer], Any] | None

_ReducedType: TypeAlias = (
    str
    | tuple[Callable[..., Any], tuple[Any, ...]]
    | tuple[Callable[..., Any], tuple[Any, ...], Any]
    | tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None]
    | tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None, Iterator[Any] | None]
)

def dump(
    obj: Any,
    file: SupportsWrite[bytes],
    protocol: int | None = None,
    *,
    fix_imports: bool = True,
    buffer_callback: _BufferCallback = None,
) -> None:
    """Write a pickled representation of obj to the open file object file.

    This is equivalent to ``Pickler(file, protocol).dump(obj)``, but may
    be more efficient.

    The optional *protocol* argument tells the pickler to use the given
    protocol; supported protocols are 0, 1, 2, 3, 4 and 5.  The default
    protocol is 5. It was introduced in Python 3.8, and is incompatible
    with previous versions.

    Specifying a negative protocol version selects the highest protocol
    version supported.  The higher the protocol used, the more recent the
    version of Python needed to read the pickle produced.

    The *file* argument must have a write() method that accepts a single
    bytes argument.  It can thus be a file object opened for binary
    writing, an io.BytesIO instance, or any other custom object that meets
    this interface.

    If *fix_imports* is True and protocol is less than 3, pickle will try
    to map the new Python 3 names to the old module names used in Python
    2, so that the pickle data stream is readable with Python 2.

    If *buffer_callback* is None (the default), buffer views are serialized
    into *file* as part of the pickle stream.  It is an error if
    *buffer_callback* is not None and *protocol* is None or smaller than 5.
    """

def dumps(obj: Any, protocol: int | None = None, *, fix_imports: bool = True, buffer_callback: _BufferCallback = None) -> bytes:
    """Return the pickled representation of the object as a bytes object.

    The optional *protocol* argument tells the pickler to use the given
    protocol; supported protocols are 0, 1, 2, 3, 4 and 5.  The default
    protocol is 5. It was introduced in Python 3.8, and is incompatible
    with previous versions.

    Specifying a negative protocol version selects the highest protocol
    version supported.  The higher the protocol used, the more recent the
    version of Python needed to read the pickle produced.

    If *fix_imports* is True and *protocol* is less than 3, pickle will
    try to map the new Python 3 names to the old module names used in
    Python 2, so that the pickle data stream is readable with Python 2.

    If *buffer_callback* is None (the default), buffer views are serialized
    into *file* as part of the pickle stream.  It is an error if
    *buffer_callback* is not None and *protocol* is None or smaller than 5.
    """

def load(
    file: _ReadableFileobj,
    *,
    fix_imports: bool = True,
    encoding: str = "ASCII",
    errors: str = "strict",
    buffers: Iterable[Any] | None = (),
) -> Any:
    """Read and return an object from the pickle data stored in a file.

    This is equivalent to ``Unpickler(file).load()``, but may be more
    efficient.

    The protocol version of the pickle is detected automatically, so no
    protocol argument is needed.  Bytes past the pickled object's
    representation are ignored.

    The argument *file* must have two methods, a read() method that takes
    an integer argument, and a readline() method that requires no
    arguments.  Both methods should return bytes.  Thus *file* can be a
    binary file object opened for reading, an io.BytesIO object, or any
    other custom object that meets this interface.

    Optional keyword arguments are *fix_imports*, *encoding* and *errors*,
    which are used to control compatibility support for pickle stream
    generated by Python 2.  If *fix_imports* is True, pickle will try to
    map the old Python 2 names to the new names used in Python 3.  The
    *encoding* and *errors* tell pickle how to decode 8-bit string
    instances pickled by Python 2; these default to 'ASCII' and 'strict',
    respectively.  The *encoding* can be 'bytes' to read these 8-bit
    string instances as bytes objects.
    """

def loads(
    data: ReadableBuffer,
    /,
    *,
    fix_imports: bool = True,
    encoding: str = "ASCII",
    errors: str = "strict",
    buffers: Iterable[Any] | None = (),
) -> Any:
    """Read and return an object from the given pickle data.

    The protocol version of the pickle is detected automatically, so no
    protocol argument is needed.  Bytes past the pickled object's
    representation are ignored.

    Optional keyword arguments are *fix_imports*, *encoding* and *errors*,
    which are used to control compatibility support for pickle stream
    generated by Python 2.  If *fix_imports* is True, pickle will try to
    map the old Python 2 names to the new names used in Python 3.  The
    *encoding* and *errors* tell pickle how to decode 8-bit string
    instances pickled by Python 2; these default to 'ASCII' and 'strict',
    respectively.  The *encoding* can be 'bytes' to read these 8-bit
    string instances as bytes objects.
    """

class PickleError(Exception): ...
class PicklingError(PickleError): ...
class UnpicklingError(PickleError): ...

@type_check_only
class PicklerMemoProxy:
    def clear(self, /) -> None: ...
    def copy(self, /) -> dict[int, tuple[int, Any]]: ...

@disjoint_base
class Pickler:
    """This takes a binary file for writing a pickle data stream.

    The optional *protocol* argument tells the pickler to use the given
    protocol; supported protocols are 0, 1, 2, 3, 4 and 5.  The default
    protocol is 5. It was introduced in Python 3.8, and is incompatible
    with previous versions.

    Specifying a negative protocol version selects the highest protocol
    version supported.  The higher the protocol used, the more recent the
    version of Python needed to read the pickle produced.

    The *file* argument must have a write() method that accepts a single
    bytes argument. It can thus be a file object opened for binary
    writing, an io.BytesIO instance, or any other custom object that meets
    this interface.

    If *fix_imports* is True and protocol is less than 3, pickle will try
    to map the new Python 3 names to the old module names used in Python
    2, so that the pickle data stream is readable with Python 2.

    If *buffer_callback* is None (the default), buffer views are
    serialized into *file* as part of the pickle stream.

    If *buffer_callback* is not None, then it can be called any number
    of times with a buffer view.  If the callback returns a false value
    (such as None), the given buffer is out-of-band; otherwise the
    buffer is serialized in-band, i.e. inside the pickle stream.

    It is an error if *buffer_callback* is not None and *protocol*
    is None or smaller than 5.
    """

    fast: bool
    dispatch_table: Mapping[type, Callable[[Any], _ReducedType]]
    reducer_override: Callable[[Any], Any]
    bin: bool  # undocumented
    def __init__(
        self,
        file: SupportsWrite[bytes],
        protocol: int | None = None,
        fix_imports: bool = True,
        buffer_callback: _BufferCallback = None,
    ) -> None: ...
    @property
    def memo(self) -> PicklerMemoProxy: ...
    @memo.setter
    def memo(self, value: PicklerMemoProxy | dict[int, tuple[int, Any]]) -> None: ...
    def dump(self, obj: Any, /) -> None:
        """Write a pickled representation of the given object to the open file."""

    def clear_memo(self) -> None:
        """Clears the pickler's "memo".

        The memo is the data structure that remembers which objects the
        pickler has already seen, so that shared or recursive objects are
        pickled by reference and not by value.  This method is useful when
        re-using picklers.
        """
    # this method has no default implementation for Python < 3.13
    def persistent_id(self, obj: Any, /) -> Any: ...

@type_check_only
class UnpicklerMemoProxy:
    def clear(self, /) -> None: ...
    def copy(self, /) -> dict[int, tuple[int, Any]]: ...

@disjoint_base
class Unpickler:
    """This takes a binary file for reading a pickle data stream.

    The protocol version of the pickle is detected automatically, so no
    protocol argument is needed.  Bytes past the pickled object's
    representation are ignored.

    The argument *file* must have two methods, a read() method that takes
    an integer argument, and a readline() method that requires no
    arguments.  Both methods should return bytes.  Thus *file* can be a
    binary file object opened for reading, an io.BytesIO object, or any
    other custom object that meets this interface.

    Optional keyword arguments are *fix_imports*, *encoding* and *errors*,
    which are used to control compatibility support for pickle stream
    generated by Python 2.  If *fix_imports* is True, pickle will try to
    map the old Python 2 names to the new names used in Python 3.  The
    *encoding* and *errors* tell pickle how to decode 8-bit string
    instances pickled by Python 2; these default to 'ASCII' and 'strict',
    respectively.  The *encoding* can be 'bytes' to read these 8-bit
    string instances as bytes objects.
    """

    def __init__(
        self,
        file: _ReadableFileobj,
        *,
        fix_imports: bool = True,
        encoding: str = "ASCII",
        errors: str = "strict",
        buffers: Iterable[Any] | None = (),
    ) -> None: ...
    @property
    def memo(self) -> UnpicklerMemoProxy: ...
    @memo.setter
    def memo(self, value: UnpicklerMemoProxy | dict[int, tuple[int, Any]]) -> None: ...
    def load(self) -> Any:
        """Load a pickle.

        Read a pickled object representation from the open file object given
        in the constructor, and return the reconstituted object hierarchy
        specified therein.
        """

    def find_class(self, module_name: str, global_name: str, /) -> Any:
        """Return an object from a specified module.

        If necessary, the module will be imported. Subclasses may override
        this method (e.g. to restrict unpickling of arbitrary classes and
        functions).

        This method is called whenever a class or a function object is
        needed.  Both arguments passed are str objects.
        """
    # this method has no default implementation for Python < 3.13
    def persistent_load(self, pid: Any, /) -> Any: ...



"""
A Path-like interface for zipfiles.

This codebase is shared between zipfile.Path in the stdlib
and zipp in PyPI. See
https://github.com/python/importlib_metadata/wiki/Development-Methodology
for more detail.
"""

import sys
from _typeshed import StrPath
from collections.abc import Iterator, Sequence
from io import TextIOWrapper
from os import PathLike
from typing import IO, Literal, TypeVar, overload
from typing_extensions import Self
from zipfile import ZipFile

_ZF = TypeVar("_ZF", bound=ZipFile)

if sys.version_info >= (3, 12):
    __all__ = ["Path"]

    class InitializedState:
        """
        Mix-in to save the initialization state for pickling.
        """

        def __init__(self, *args: object, **kwargs: object) -> None: ...
        def __getstate__(self) -> tuple[list[object], dict[object, object]]: ...
        def __setstate__(self, state: Sequence[tuple[list[object], dict[object, object]]]) -> None: ...

    class CompleteDirs(InitializedState, ZipFile):
        """
        A ZipFile subclass that ensures that implied directories
        are always included in the namelist.

        >>> list(CompleteDirs._implied_dirs(['foo/bar.txt', 'foo/bar/baz.txt']))
        ['foo/', 'foo/bar/']
        >>> list(CompleteDirs._implied_dirs(['foo/bar.txt', 'foo/bar/baz.txt', 'foo/bar/']))
        ['foo/']
        """

        def resolve_dir(self, name: str) -> str:
            """
            If the name represents a directory, return that name
            as a directory (with the trailing slash).
            """

        @overload
        @classmethod
        def make(cls, source: ZipFile) -> CompleteDirs:
            """
            Given a source (filename or zipfile), return an
            appropriate CompleteDirs subclass.
            """

        @overload
        @classmethod
        def make(cls, source: StrPath | IO[bytes]) -> Self: ...
        if sys.version_info >= (3, 13):
            @classmethod
            def inject(cls, zf: _ZF) -> _ZF:
                """
                Given a writable zip file zf, inject directory entries for
                any directories implied by the presence of children.
                """

    class Path:
        """
        A :class:`importlib.resources.abc.Traversable` interface for zip files.

        Implements many of the features users enjoy from
        :class:`pathlib.Path`.

        Consider a zip file with this structure::

            .
            ├── a.txt
            └── b
                ├── c.txt
                └── d
                    └── e.txt

        >>> data = io.BytesIO()
        >>> zf = ZipFile(data, 'w')
        >>> zf.writestr('a.txt', 'content of a')
        >>> zf.writestr('b/c.txt', 'content of c')
        >>> zf.writestr('b/d/e.txt', 'content of e')
        >>> zf.filename = 'mem/abcde.zip'

        Path accepts the zipfile object itself or a filename

        >>> path = Path(zf)

        From there, several path operations are available.

        Directory iteration (including the zip file itself):

        >>> a, b = path.iterdir()
        >>> a
        Path('mem/abcde.zip', 'a.txt')
        >>> b
        Path('mem/abcde.zip', 'b/')

        name property:

        >>> b.name
        'b'

        join with divide operator:

        >>> c = b / 'c.txt'
        >>> c
        Path('mem/abcde.zip', 'b/c.txt')
        >>> c.name
        'c.txt'

        Read text:

        >>> c.read_text(encoding='utf-8')
        'content of c'

        existence:

        >>> c.exists()
        True
        >>> (b / 'missing.txt').exists()
        False

        Coercion to string:

        >>> import os
        >>> str(c).replace(os.sep, posixpath.sep)
        'mem/abcde.zip/b/c.txt'

        At the root, ``name``, ``filename``, and ``parent``
        resolve to the zipfile.

        >>> str(path)
        'mem/abcde.zip/'
        >>> path.name
        'abcde.zip'
        >>> path.filename == pathlib.Path('mem/abcde.zip')
        True
        >>> str(path.parent)
        'mem'

        If the zipfile has no filename, such attributes are not
        valid and accessing them will raise an Exception.

        >>> zf.filename = None
        >>> path.name
        Traceback (most recent call last):
        ...
        TypeError: ...

        >>> path.filename
        Traceback (most recent call last):
        ...
        TypeError: ...

        >>> path.parent
        Traceback (most recent call last):
        ...
        TypeError: ...

        # workaround python/cpython#106763
        >>> pass
        """

        root: CompleteDirs
        at: str
        def __init__(self, root: ZipFile | StrPath | IO[bytes], at: str = "") -> None:
            """
            Construct a Path from a ZipFile or filename.

            Note: When the source is an existing ZipFile object,
            its type (__class__) will be mutated to a
            specialized type. If the caller wishes to retain the
            original type, the caller should either create a
            separate ZipFile object or pass a filename.
            """

        @property
        def name(self) -> str: ...
        @property
        def parent(self) -> PathLike[str]: ...  # undocumented
        @property
        def filename(self) -> PathLike[str]: ...  # undocumented
        @property
        def suffix(self) -> str: ...
        @property
        def suffixes(self) -> list[str]: ...
        @property
        def stem(self) -> str: ...
        @overload
        def open(
            self,
            mode: Literal["r", "w"] = "r",
            encoding: str | None = None,
            errors: str | None = None,
            newline: str | None = None,
            line_buffering: bool = False,
            write_through: bool = False,
            *,
            pwd: bytes | None = None,
        ) -> TextIOWrapper:
            """
            Open this entry as text or binary following the semantics
            of ``pathlib.Path.open()`` by passing arguments through
            to io.TextIOWrapper().
            """

        @overload
        def open(self, mode: Literal["rb", "wb"], *, pwd: bytes | None = None) -> IO[bytes]: ...
        def iterdir(self) -> Iterator[Self]: ...
        def is_dir(self) -> bool: ...
        def is_file(self) -> bool: ...
        def exists(self) -> bool: ...
        def read_text(
            self,
            encoding: str | None = None,
            errors: str | None = None,
            newline: str | None = None,
            line_buffering: bool = False,
            write_through: bool = False,
        ) -> str: ...
        def read_bytes(self) -> bytes: ...
        def joinpath(self, *other: StrPath) -> Path: ...
        def glob(self, pattern: str) -> Iterator[Self]: ...
        def rglob(self, pattern: str) -> Iterator[Self]: ...
        def is_symlink(self) -> Literal[False]:
            """
            Return whether this path is a symlink.
            """

        def relative_to(self, other: Path, *extra: StrPath) -> str: ...
        def match(self, path_pattern: str) -> bool: ...
        def __eq__(self, other: object) -> bool:
            """
            >>> Path(zipfile.ZipFile(io.BytesIO(), 'w')) == 'foo'
            False
            """

        def __hash__(self) -> int: ...
        def __truediv__(self, add: StrPath) -> Path: ...



"""distutils.dep_util

Utility functions for simple, timestamp-based dependency of files
and groups of files; also, function based entirely on such
timestamp dependency analysis.
"""

from _typeshed import StrOrBytesPath, SupportsLenAndGetItem
from collections.abc import Iterable
from typing import Literal, TypeVar

_SourcesT = TypeVar("_SourcesT", bound=StrOrBytesPath)
_TargetsT = TypeVar("_TargetsT", bound=StrOrBytesPath)

def newer(source: StrOrBytesPath, target: StrOrBytesPath) -> bool | Literal[1]:
    """Return true if 'source' exists and is more recently modified than
    'target', or if 'source' exists and 'target' doesn't.  Return false if
    both exist and 'target' is the same age or younger than 'source'.
    Raise DistutilsFileError if 'source' does not exist.
    """

def newer_pairwise(
    sources: SupportsLenAndGetItem[_SourcesT], targets: SupportsLenAndGetItem[_TargetsT]
) -> tuple[list[_SourcesT], list[_TargetsT]]:
    """Walk two filename lists in parallel, testing if each source is newer
    than its corresponding target.  Return a pair of lists (sources,
    targets) where source is newer than target, according to the semantics
    of 'newer()'.
    """

def newer_group(
    sources: Iterable[StrOrBytesPath], target: StrOrBytesPath, missing: Literal["error", "ignore", "newer"] = "error"
) -> Literal[0, 1]:
    """Return true if 'target' is out-of-date with respect to any file
    listed in 'sources'.  In other words, if 'target' exists and is newer
    than every file in 'sources', return false; otherwise return true.
    'missing' controls what we do when a source file is missing; the
    default ("error") is to blow up with an OSError from inside 'stat()';
    if it is "ignore", we silently drop any missing source files; if it is
    "newer", any missing source files make us assume that 'target' is
    out-of-date (this is handy in "dry-run" mode: it'll make you pretend to
    carry out commands that wouldn't work because inputs are missing, but
    that doesn't matter because you're not actually going to run the
    commands).
    """



"""Interface to the Expat non-validating XML parser."""

from pyexpat import *

# This is actually implemented in the C module pyexpat, but considers itself to live here.
class ExpatError(Exception):
    code: int
    lineno: int
    offset: int



"""Interface to the liblzma compression library.

This module provides a class for reading and writing compressed files,
classes for incremental (de)compression, and convenience functions for
one-shot (de)compression.

These classes and functions support both the XZ and legacy LZMA
container formats, as well as raw compressed data streams.
"""

from lzma import *



# flags: --line-ranges=8-8
# NOTE: If you need to modify this file, pay special attention to the --line-ranges=
# flag above as it's formatting specifically these lines.

# This test ensures no empty lines are added around import lines.
# It caused an issue before https://github.com/psf/black/pull/3610 is merged.
import os
import re
import sys



# Formats the entire function with tab or 4 space indentation
# because the statement indentations don't match the preferred indentation.
def test  ():
  print("before" )
  <RANGE_START>1 +  2
  if   True:
      pass
  print("Done" )<RANGE_END>

  print("formatted" )

print("not formatted" )

def test2  ():
  print("before" )
  <RANGE_START>1 +  2
  (
3 + 2
  )
  print("Done" )<RANGE_END>

  print("formatted" )

print("not formatted" )

def test3  ():
  print("before" )
  <RANGE_START>1 +  2
  """A Multiline string
that starts at the beginning of the line and we need to preserve the leading spaces"""

  """A Multiline string
  that has some indentation on the second line and we need to preserve the leading spaces"""

  print("Done" )<RANGE_END>


def test4  ():
  print("before" )
  <RANGE_START>1 +  2
  """A Multiline string
    that uses the same indentation as the formatted code will. This should not be dedented."""

  print("Done" )<RANGE_END>

def test5 ():
  print("before" )
  if True:
      print("Format to fix indentation" )
      print(<RANGE_START>1 +  2)

  else:
      print(3 +  4)<RANGE_END>
      print("Format to fix indentation" )

  pass


def test6 ():
    <RANGE_START>
    print("Format" )
    print(3 +  4)<RANGE_END>
    print("Format to fix indentation" )


def test7 ():
    <RANGE_START>print("Format" )
    print(3 +  4)<RANGE_END>
    print("Format to fix indentation" )



"""distutils.fancy_getopt

Wrapper around the standard getopt module that provides the following
additional features:
  * short and long options are tied together
  * options have help strings, so fancy_getopt could potentially
    create a complete usage summary
  * options set attributes of a passed-in object
"""

from collections.abc import Iterable, Mapping
from getopt import _SliceableT, _StrSequenceT_co
from re import Pattern
from typing import Any, Final, overload
from typing_extensions import TypeAlias

_Option: TypeAlias = tuple[str, str | None, str]

longopt_pat: Final = r"[a-zA-Z](?:[a-zA-Z0-9-]*)"
longopt_re: Final[Pattern[str]]
neg_alias_re: Final[Pattern[str]]
longopt_xlate: Final[dict[int, int]]

class FancyGetopt:
    """Wrapper around the standard 'getopt()' module that provides some
    handy extra functionality:
      * short and long options are tied together
      * options have help strings, and help text can be assembled
        from them
      * options set attributes of a passed-in object
      * boolean options can have "negative aliases" -- eg. if
        --quiet is the "negative alias" of --verbose, then "--quiet"
        on the command line sets 'verbose' to false
    """

    def __init__(self, option_table: list[_Option] | None = None) -> None: ...
    # TODO: kinda wrong, `getopt(object=object())` is invalid
    @overload
    def getopt(
        self, args: _SliceableT[_StrSequenceT_co] | None = None, object: None = None
    ) -> tuple[_StrSequenceT_co, OptionDummy]:
        """Parse command-line options in args. Store as attributes on object.

        If 'args' is None or not supplied, uses 'sys.argv[1:]'.  If
        'object' is None or not supplied, creates a new OptionDummy
        object, stores option values there, and returns a tuple (args,
        object).  If 'object' is supplied, it is modified in place and
        'getopt()' just returns 'args'; in both cases, the returned
        'args' is a modified copy of the passed-in 'args' list, which
        is left untouched.
        """

    @overload
    def getopt(
        self, args: _SliceableT[_StrSequenceT_co] | None, object: Any
    ) -> _StrSequenceT_co: ...  # object is an arbitrary non-slotted object
    def get_option_order(self) -> list[tuple[str, str]]:
        """Returns the list of (option, value) tuples processed by the
        previous run of 'getopt()'.  Raises RuntimeError if
        'getopt()' hasn't been called yet.
        """

    def generate_help(self, header: str | None = None) -> list[str]:
        """Generate help text (a list of strings, one per suggested line of
        output) from the option table for this FancyGetopt object.
        """

# Same note as FancyGetopt.getopt
@overload
def fancy_getopt(
    options: list[_Option], negative_opt: Mapping[_Option, _Option], object: None, args: _SliceableT[_StrSequenceT_co] | None
) -> tuple[_StrSequenceT_co, OptionDummy]: ...
@overload
def fancy_getopt(
    options: list[_Option], negative_opt: Mapping[_Option, _Option], object: Any, args: _SliceableT[_StrSequenceT_co] | None
) -> _StrSequenceT_co: ...

WS_TRANS: Final[dict[int, str]]

def wrap_text(text: str, width: int) -> list[str]:
    """wrap_text(text : string, width : int) -> [string]

    Split 'text' into multiple lines of no more than 'width' characters
    each, and return the list of strings that results.
    """

def translate_longopt(opt: str) -> str:
    """Convert a long option name to a valid Python identifier by
    changing "-" to "_".
    """

class OptionDummy:
    """Dummy class just used as a place to hold command-line option
    values as instance attributes.
    """

    def __init__(self, options: Iterable[str] = []) -> None:
        """Create a new OptionDummy instance.  The attributes listed in
        'options' will be initialized to None.
        """



from a import aksjdhflsakhdflkjsadlfajkslhf
from a import (
    aksjdhflsakhdflkjsadlfajkslhf,
)
from a import (
    aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa,
)
from a import (
    aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa,
    aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa,
)
from a import (
    aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa as dfgsdfgsd,
    aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa as sdkjflsdjlahlfd,
)
from aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa import *


from a import bar  # comment

from a import bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar  # comment

from a import (  # comment
    bar,
)

from a import (  # comment
    bar
)

from a import bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar, bar
# comment

from a import \
    (  # comment
    bar,
)

from tqdm .  auto import tqdm



import enum
import sys
import types
from _typeshed import DataclassInstance
from builtins import type as Type  # alias to avoid name clashes with fields named "type"
from collections.abc import Callable, Iterable, Mapping
from types import GenericAlias
from typing import Any, Final, Generic, Literal, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Never, TypeIs

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)

__all__ = [
    "dataclass",
    "field",
    "Field",
    "FrozenInstanceError",
    "InitVar",
    "MISSING",
    "fields",
    "asdict",
    "astuple",
    "make_dataclass",
    "replace",
    "is_dataclass",
]

if sys.version_info >= (3, 10):
    __all__ += ["KW_ONLY"]

_DataclassT = TypeVar("_DataclassT", bound=DataclassInstance)

@type_check_only
class _DataclassFactory(Protocol):
    def __call__(
        self,
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type[_T]: ...

# define _MISSING_TYPE as an enum within the type stubs,
# even though that is not really its type at runtime
# this allows us to use Literal[_MISSING_TYPE.MISSING]
# for background, see:
#   https://github.com/python/typeshed/pull/5900#issuecomment-895513797
class _MISSING_TYPE(enum.Enum):
    MISSING = enum.auto()

MISSING: Final = _MISSING_TYPE.MISSING

if sys.version_info >= (3, 10):
    class KW_ONLY: ...

@overload
def asdict(obj: DataclassInstance) -> dict[str, Any]:
    """Return the fields of a dataclass instance as a new dictionary mapping
    field names to field values.

    Example usage::

      @dataclass
      class C:
          x: int
          y: int

      c = C(1, 2)
      assert asdict(c) == {'x': 1, 'y': 2}

    If given, 'dict_factory' will be used instead of built-in dict.
    The function applies recursively to field values that are
    dataclass instances. This will also look into built-in containers:
    tuples, lists, and dicts. Other objects are copied with 'copy.deepcopy()'.
    """

@overload
def asdict(obj: DataclassInstance, *, dict_factory: Callable[[list[tuple[str, Any]]], _T]) -> _T: ...
@overload
def astuple(obj: DataclassInstance) -> tuple[Any, ...]:
    """Return the fields of a dataclass instance as a new tuple of field values.

    Example usage::

      @dataclass
      class C:
          x: int
          y: int

      c = C(1, 2)
      assert astuple(c) == (1, 2)

    If given, 'tuple_factory' will be used instead of built-in tuple.
    The function applies recursively to field values that are
    dataclass instances. This will also look into built-in containers:
    tuples, lists, and dicts. Other objects are copied with 'copy.deepcopy()'.
    """

@overload
def astuple(obj: DataclassInstance, *, tuple_factory: Callable[[list[Any]], _T]) -> _T: ...

if sys.version_info >= (3, 11):
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type[_T]:
        """Add dunder methods based on the fields defined in the class.

        Examines PEP 526 __annotations__ to determine fields.

        If init is true, an __init__() method is added to the class. If repr
        is true, a __repr__() method is added. If order is true, rich
        comparison dunder methods are added. If unsafe_hash is true, a
        __hash__() method is added. If frozen is true, fields may not be
        assigned to after instance creation. If match_args is true, the
        __match_args__ tuple is added. If kw_only is true, then by default
        all fields are keyword-only. If slots is true, a new class with a
        __slots__ attribute is returned.
        """

    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

elif sys.version_info >= (3, 10):
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> type[_T]:
        """Returns the same class as was passed in, with dunder methods
        added based on the fields defined in the class.

        Examines PEP 526 __annotations__ to determine fields.

        If init is true, an __init__() method is added to the class. If
        repr is true, a __repr__() method is added. If order is true, rich
        comparison dunder methods are added. If unsafe_hash is true, a
        __hash__() method function is added. If frozen is true, fields may
        not be assigned to after instance creation. If match_args is true,
        the __match_args__ tuple is added. If kw_only is true, then by
        default all fields are keyword-only. If slots is true, an
        __slots__ attribute is added.
        """

    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

else:
    @overload
    def dataclass(
        cls: type[_T],
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> type[_T]:
        """Returns the same class as was passed in, with dunder methods
        added based on the fields defined in the class.

        Examines PEP 526 __annotations__ to determine fields.

        If init is true, an __init__() method is added to the class. If
        repr is true, a __repr__() method is added. If order is true, rich
        comparison dunder methods are added. If unsafe_hash is true, a
        __hash__() method function is added. If frozen is true, fields may
        not be assigned to after instance creation.
        """

    @overload
    def dataclass(
        cls: None = None,
        /,
        *,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> Callable[[type[_T]], type[_T]]: ...

# See https://github.com/python/mypy/issues/10750
@type_check_only
class _DefaultFactory(Protocol[_T_co]):
    def __call__(self) -> _T_co: ...

class Field(Generic[_T]):
    if sys.version_info >= (3, 14):
        __slots__ = (
            "name",
            "type",
            "default",
            "default_factory",
            "repr",
            "hash",
            "init",
            "compare",
            "metadata",
            "kw_only",
            "doc",
            "_field_type",
        )
    elif sys.version_info >= (3, 10):
        __slots__ = (
            "name",
            "type",
            "default",
            "default_factory",
            "repr",
            "hash",
            "init",
            "compare",
            "metadata",
            "kw_only",
            "_field_type",
        )
    else:
        __slots__ = ("name", "type", "default", "default_factory", "repr", "hash", "init", "compare", "metadata", "_field_type")
    name: str
    type: Type[_T] | str | Any
    default: _T | Literal[_MISSING_TYPE.MISSING]
    default_factory: _DefaultFactory[_T] | Literal[_MISSING_TYPE.MISSING]
    repr: bool
    hash: bool | None
    init: bool
    compare: bool
    metadata: types.MappingProxyType[Any, Any]

    if sys.version_info >= (3, 14):
        doc: str | None

    if sys.version_info >= (3, 10):
        kw_only: bool | Literal[_MISSING_TYPE.MISSING]

    if sys.version_info >= (3, 14):
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
            kw_only: bool,
            doc: str | None,
        ) -> None: ...
    elif sys.version_info >= (3, 10):
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
            kw_only: bool,
        ) -> None: ...
    else:
        def __init__(
            self,
            default: _T,
            default_factory: Callable[[], _T],
            init: bool,
            repr: bool,
            hash: bool | None,
            compare: bool,
            metadata: Mapping[Any, Any],
        ) -> None: ...

    def __set_name__(self, owner: Type[Any], name: str) -> None: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """Represent a PEP 585 generic type

        E.g. for t = list[int], t.__origin__ is list and t.__args__ is (int,).
        """

# NOTE: Actual return type is 'Field[_T]', but we want to help type checkers
# to understand the magic that happens at runtime.
if sys.version_info >= (3, 14):
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> _T:
        """Return an object to identify dataclass fields.

        default is the default value of the field.  default_factory is a
        0-argument function called to initialize a field's value.  If init
        is true, the field will be a parameter to the class's __init__()
        function.  If repr is true, the field will be included in the
        object's repr().  If hash is true, the field will be included in the
        object's hash().  If compare is true, the field will be used in
        comparison functions.  metadata, if specified, must be a mapping
        which is stored but not otherwise examined by dataclass.  If kw_only
        is true, the field will become a keyword-only parameter to
        __init__().  doc is an optional docstring for this field.

        It is an error to specify both default and default_factory.
        """

    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
        doc: str | None = None,
    ) -> Any: ...

elif sys.version_info >= (3, 10):
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> _T:
        """Return an object to identify dataclass fields.

        default is the default value of the field.  default_factory is a
        0-argument function called to initialize a field's value.  If init
        is true, the field will be a parameter to the class's __init__()
        function.  If repr is true, the field will be included in the
        object's repr().  If hash is true, the field will be included in the
        object's hash().  If compare is true, the field will be used in
        comparison functions.  metadata, if specified, must be a mapping
        which is stored but not otherwise examined by dataclass.  If kw_only
        is true, the field will become a keyword-only parameter to
        __init__().

        It is an error to specify both default and default_factory.
        """

    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
        kw_only: bool | Literal[_MISSING_TYPE.MISSING] = ...,
    ) -> Any: ...

else:
    @overload  # `default` and `default_factory` are optional and mutually exclusive.
    def field(
        *,
        default: _T,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> _T:
        """Return an object to identify dataclass fields.

        default is the default value of the field.  default_factory is a
        0-argument function called to initialize a field's value.  If init
        is True, the field will be a parameter to the class's __init__()
        function.  If repr is True, the field will be included in the
        object's repr().  If hash is True, the field will be included in
        the object's hash().  If compare is True, the field will be used
        in comparison functions.  metadata, if specified, must be a
        mapping which is stored but not otherwise examined by dataclass.

        It is an error to specify both default and default_factory.
        """

    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Callable[[], _T],
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> _T: ...
    @overload
    def field(
        *,
        default: Literal[_MISSING_TYPE.MISSING] = ...,
        default_factory: Literal[_MISSING_TYPE.MISSING] = ...,
        init: bool = True,
        repr: bool = True,
        hash: bool | None = None,
        compare: bool = True,
        metadata: Mapping[Any, Any] | None = None,
    ) -> Any: ...

def fields(class_or_instance: DataclassInstance | type[DataclassInstance]) -> tuple[Field[Any], ...]:
    """Return a tuple describing the fields of this dataclass.

    Accepts a dataclass or an instance of one. Tuple elements are of
    type Field.
    """

# HACK: `obj: Never` typing matches if object argument is using `Any` type.
@overload
def is_dataclass(obj: Never) -> TypeIs[DataclassInstance | type[DataclassInstance]]:  # type: ignore[narrowed-type-not-subtype]  # pyright: ignore[reportGeneralTypeIssues]
    """Returns True if obj is a dataclass or an instance of a
    dataclass.
    """

@overload
def is_dataclass(obj: type) -> TypeIs[type[DataclassInstance]]: ...
@overload
def is_dataclass(obj: object) -> TypeIs[DataclassInstance | type[DataclassInstance]]: ...

class FrozenInstanceError(AttributeError): ...

class InitVar(Generic[_T]):
    __slots__ = ("type",)
    type: Type[_T]
    def __init__(self, type: Type[_T]) -> None: ...
    @overload
    def __class_getitem__(cls, type: Type[_T]) -> InitVar[_T]: ...  # pyright: ignore[reportInvalidTypeForm]
    @overload
    def __class_getitem__(cls, type: Any) -> InitVar[Any]: ...  # pyright: ignore[reportInvalidTypeForm]

if sys.version_info >= (3, 14):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
        module: str | None = None,
        decorator: _DataclassFactory = ...,
    ) -> type:
        """Return a new dynamically created dataclass.

        The dataclass name will be 'cls_name'.  'fields' is an iterable
        of either (name), (name, type) or (name, type, Field) objects. If type is
        omitted, use the string 'typing.Any'.  Field objects are created by
        the equivalent of calling 'field(name, type [, Field-info])'.::

          C = make_dataclass('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))

        is equivalent to::

          @dataclass
          class C(Base):
              x: 'typing.Any'
              y: int
              z: int = field(init=False)

        For the bases and namespace parameters, see the builtin type() function.

        The parameters init, repr, eq, order, unsafe_hash, frozen, match_args, kw_only,
        slots, and weakref_slot are passed to dataclass().

        If module parameter is defined, the '__module__' attribute of the dataclass is
        set to that value.
        """

elif sys.version_info >= (3, 12):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
        module: str | None = None,
    ) -> type:
        """Return a new dynamically created dataclass.

        The dataclass name will be 'cls_name'.  'fields' is an iterable
        of either (name), (name, type) or (name, type, Field) objects. If type is
        omitted, use the string 'typing.Any'.  Field objects are created by
        the equivalent of calling 'field(name, type [, Field-info])'.::

          C = make_dataclass('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))

        is equivalent to::

          @dataclass
          class C(Base):
              x: 'typing.Any'
              y: int
              z: int = field(init=False)

        For the bases and namespace parameters, see the builtin type() function.

        The parameters init, repr, eq, order, unsafe_hash, frozen, match_args, kw_only,
        slots, and weakref_slot are passed to dataclass().

        If module parameter is defined, the '__module__' attribute of the dataclass is
        set to that value.
        """

elif sys.version_info >= (3, 11):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
        weakref_slot: bool = False,
    ) -> type:
        """Return a new dynamically created dataclass.

        The dataclass name will be 'cls_name'.  'fields' is an iterable
        of either (name), (name, type) or (name, type, Field) objects. If type is
        omitted, use the string 'typing.Any'.  Field objects are created by
        the equivalent of calling 'field(name, type [, Field-info])'.::

          C = make_dataclass('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))

        is equivalent to::

          @dataclass
          class C(Base):
              x: 'typing.Any'
              y: int
              z: int = field(init=False)

        For the bases and namespace parameters, see the builtin type() function.

        The parameters init, repr, eq, order, unsafe_hash, and frozen are passed to
        dataclass().
        """

elif sys.version_info >= (3, 10):
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
        match_args: bool = True,
        kw_only: bool = False,
        slots: bool = False,
    ) -> type:
        """Return a new dynamically created dataclass.

        The dataclass name will be 'cls_name'.  'fields' is an iterable
        of either (name), (name, type) or (name, type, Field) objects. If type is
        omitted, use the string 'typing.Any'.  Field objects are created by
        the equivalent of calling 'field(name, type [, Field-info])'.

          C = make_dataclass('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))

        is equivalent to:

          @dataclass
          class C(Base):
              x: 'typing.Any'
              y: int
              z: int = field(init=False)

        For the bases and namespace parameters, see the builtin type() function.

        The parameters init, repr, eq, order, unsafe_hash, and frozen are passed to
        dataclass().
        """

else:
    def make_dataclass(
        cls_name: str,
        fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
        *,
        bases: tuple[type, ...] = (),
        namespace: dict[str, Any] | None = None,
        init: bool = True,
        repr: bool = True,
        eq: bool = True,
        order: bool = False,
        unsafe_hash: bool = False,
        frozen: bool = False,
    ) -> type:
        """Return a new dynamically created dataclass.

        The dataclass name will be 'cls_name'.  'fields' is an iterable
        of either (name), (name, type) or (name, type, Field) objects. If type is
        omitted, use the string 'typing.Any'.  Field objects are created by
        the equivalent of calling 'field(name, type [, Field-info])'.

          C = make_dataclass('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))

        is equivalent to:

          @dataclass
          class C(Base):
              x: 'typing.Any'
              y: int
              z: int = field(init=False)

        For the bases and namespace parameters, see the builtin type() function.

        The parameters init, repr, eq, order, unsafe_hash, and frozen are passed to
        dataclass().
        """

def replace(obj: _DataclassT, /, **changes: Any) -> _DataclassT:
    """Return a new object replacing specified fields with new values.

    This is especially useful for frozen classes.  Example usage::

      @dataclass(frozen=True)
      class C:
          x: int
          y: int

      c = C(1, 2)
      c1 = replace(c, x=3)
      assert c1.x == 3 and c1.y == 2
    """
